////////////////////////////////////////////////////
// nopeustestim_testaaja_v02.c
////////////////////////////////////////////////////
// Copyright 2019-2020 Jaakko Kairus
// jaakko@kair.us
//
// You may freely use and modify this software for personal use.
// Commercial use of this software or parts of it is prohibited
// without written permission from author.
// DISCLAIMER: This software comes with ABSOLUTELY NO WARRANTY
//
// For more information about this project see project web page at
// http://kair.us/projects/nopeustesti/index.html
//
// Compiled with CCS compiler version 5.092
// Total compiled length 951 words 
///////////////////////////////////////////////////
// Version history
//
// 18.3.2019 initial version 0.1
//
// 17.5.2020 v0.2
// Disabled weak pull-ups from RA0..3 because they were preventing the 10k
// resistor to pull the input low enough. Also changed RA0..3 input levels
// to TTL levels to properly recognize high level.



#include <nopeustestin_testaaja.h>

#int_timer1
void timer1isr(void) {
   set_timer1(64536);   // 1000 cycles before overflow - sets interrupt rate to approx 4 kHz

   if (debounce)
      debounce--;
   else if (new_press==0) {
      last_button=current_button;
      current_button=BUT_NONE;
      if (!input(BLUE_BUTTON))   // blue
         current_button=BUT_BLUE;
      if (!input(YELLOW_BUTTON))   // yellow
         current_button=BUT_YELLOW;
      if (!input(GREEN_BUTTON))   // green
         current_button=BUT_GREEN;
      if (!input(RED_BUTTON))   // red
         current_button=BUT_RED;
      if (current_button<BUT_NONE && current_button!=last_button) {
         debounce = 20;             // 20 means 5 millisecond debounce, because interrupt rate is 250 s
         new_press=1;
      }
   }
}


void main(void) {
   
   set_input_level_a(0b11110000);    // Set RA0..3 to use TTL input levels
   set_tris_a(0b00001111);
   set_tris_b(0b10000000);
   set_tris_c(0x00);
   set_tris_e(0b11111000);
   port_a_pullups(0b00000000);       // Disable RA0..3 weak pull-ups
//   port_a_pullups(0b00001111);       // Enable RA0..3 weak pull-ups
   port_b_pullups(0b11000000);
   port_e_pullups(0b00001000);
   output_a(0x00);
   output_b(0x00);
   output_c(0x00);

   setup_timer_1(T1_INTERNAL|T1_DIV_BY_1);   // INTERNAL means Fosc/4. With 16 MHz Fosc, these settings mean 16.384 ms period
                                             // Timer1 increments every 250 ns.
   enable_interrupts(INT_TIMER1);
   enable_interrupts(GLOBAL);

   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_64); // set timer0 to maximum of 4s
                                             // INTERNAL means Fosc/4. Prescaler 64. So timer0 increments every 16 s.
   new_press = 0;

   printf("Nopeustestin testaaja v0.2\n\r");
   delay_ms(1000);

   unsigned int16 aika;
   unsigned int16 tulos;
   BUTTONS buffered_button;
   
   while (TRUE) {
      if (new_press) {
         aika = get_timer0();
         set_timer0(0);
         //printf("curr=%i, buff=%i\r\n",current_button,buffered_button);
         if (TMR0IF) {
            printf("Testi alkaa\r\n");
            printf("Pisteet,Viive,Nappi\r\n");
            aika = 0;
            tulos = 0;
            buffered_button = BUT_NONE;
         }
         else {
            tulos++;
            litbutton(buffered_button);
            // delay_ms(1);
            printf("%Lu,%Lu,%i\n\r",tulos,aika,buffered_button);
            delay_ms(10);
            alloff();
         }
         buffered_button = current_button;
         TMR0IF = 0;
         new_press=0;
      }
   }   // END OF ENDLESS LOOP
}   // END OF MAIN


void alloff(void) {
   LAMP_OFF(BLUE_LAMP);
   LAMP_OFF(YELLOW_LAMP);
   LAMP_OFF(GREEN_LAMP);
   LAMP_OFF(RED_LAMP);
}

void litbutton(int color) {
   LAMP_ON(lamp[color]);
}


